@@ -3,6 +3,8 @@ from __future__ import annotations
 
 import asyncio
 import datetime
+import os
+import ssl
 import urllib.parse
 from collections.abc import Mapping
 from json import JSONDecodeError
@@ -12,13 +14,20 @@ from typing import Any, cast
 import backoff
 import boto3
 import humps
-from aiohttp import ClientResponse, ClientSession, FormData, ServerDisconnectedError
+from aiohttp import (
+    ClientConnectorError,
+    ClientResponse,
+    ClientSession,
+    FormData,
+    ServerDisconnectedError,
+)
 from botocore.config import Config
 from warrant_lite import WarrantLite
 
 from pyoverkiz.const import (
     COZYTOUCH_ATLANTIC_API,
     COZYTOUCH_CLIENT_ID,
+    LOCAL_API_PATH,
     NEXITY_API,
     NEXITY_COGNITO_CLIENT_ID,
     NEXITY_COGNITO_REGION,
@@ -28,11 +37,13 @@ from pyoverkiz.const import (
     SOMFY_CLIENT_SECRET,
     SUPPORTED_SERVERS,
 )
+from pyoverkiz.enums import APIType, Server
 from pyoverkiz.exceptions import (
     AccessDeniedToGatewayException,
     BadCredentialsException,
     CozyTouchBadCredentialsException,
     CozyTouchServiceException,
+    ExecutionQueueFullException,
     InvalidCommandException,
     InvalidEventListenerIdException,
     InvalidTokenException,
@@ -43,6 +54,8 @@ from pyoverkiz.exceptions import (
     NoRegisteredEventListenerException,
     NotAuthenticatedException,
     NotSuchTokenException,
+    OverkizException,
+    ServiceUnavailableException,
     SessionAndBearerInSameRequestException,
     SomfyBadCredentialsException,
     SomfyServiceException,
@@ -61,6 +74,8 @@ from pyoverkiz.models import (
     Gateway,
     HistoryExecution,
     LocalToken,
+    Option,
+    OptionParameter,
     OverkizServer,
     Place,
     Scenario,
@@ -93,10 +108,12 @@ class OverkizClient:
     gateways: list[Gateway]
     event_listener_id: str | None
     session: ClientSession
+    api_type: APIType
 
     _refresh_token: str | None = None
     _expires_in: datetime.datetime | None = None
     _access_token: str | None = None
+    _ssl_context: ssl.SSLContext | None = None
 
     def __init__(
         self,
@@ -105,6 +122,7 @@ class OverkizClient:
         server: OverkizServer,
         token: str | None = None,
         session: ClientSession | None = None,
+        verify_ssl: bool | None = True,
     ) -> None:
         """
         Constructor
@@ -127,6 +145,19 @@ class OverkizClient:
 
         self.session = session if session else ClientSession()
 
+        if LOCAL_API_PATH in self.server.endpoint:
+            self.api_type = APIType.LOCAL
+
+            if verify_ssl:
+                # To avoid security issues, we add the following authority to
+                # our HTTPS client trust store: https://ca.overkiz.com/overkiz-root-ca-2048.crt
+                self._ssl_context = ssl.create_default_context(
+                    cafile=os.path.dirname(os.path.realpath(__file__))
+                    + "/overkiz-root-ca-2048.crt"
+                )
+        else:
+            self.api_type = APIType.CLOUD
+
     async def __aenter__(self) -> OverkizClient:
         return self
 
@@ -154,17 +185,18 @@ class OverkizClient:
         Caller must provide one of [userId+userPassword, userId+ssoToken, accessToken, jwt]
         """
         # Local authentication
-        if "/enduser-mobile-web/1/enduserAPI/" in self.server.endpoint:
+        if self.api_type == APIType.LOCAL:
             if register_event_listener:
                 await self.register_event_listener()
             else:
                 # Call a simple endpoint to verify if our token is correct
+                # Since local API does not have a /login endpoint
                 await self.get_gateways()
 
             return True
 
         # Somfy TaHoma authentication using access_token
-        if self.server == SUPPORTED_SERVERS["somfy_europe"]:
+        if self.server == SUPPORTED_SERVERS[Server.SOMFY_EUROPE]:
             await self.somfy_tahoma_get_access_token()
 
             if register_event_listener:
@@ -173,12 +205,12 @@ class OverkizClient:
             return True
 
         # CozyTouch authentication using jwt
-        if self.server == SUPPORTED_SERVERS["atlantic_cozytouch"]:
+        if self.server == SUPPORTED_SERVERS[Server.ATLANTIC_COZYTOUCH]:
             jwt = await self.cozytouch_login()
             payload = {"jwt": jwt}
 
         # Nexity authentication using ssoToken
-        elif self.server == SUPPORTED_SERVERS["nexity"]:
+        elif self.server == SUPPORTED_SERVERS[Server.NEXITY]:
             sso_token = await self.nexity_login()
             user_id = self.username.replace("@", "_-_")  # Replace @ for _-_
             payload = {"ssoToken": sso_token, "userId": user_id}
@@ -237,7 +269,7 @@ class OverkizClient:
         """
         Update the access and the refresh token. The refresh token will be valid 14 days.
         """
-        if self.server != SUPPORTED_SERVERS["somfy_europe"]:
+        if self.server != SUPPORTED_SERVERS[Server.SOMFY_EUROPE]:
             return
 
         if not self._refresh_token:
@@ -360,7 +392,7 @@ class OverkizClient:
 
     @backoff.on_exception(
         backoff.expo,
-        (NotAuthenticatedException, ServerDisconnectedError),
+        (NotAuthenticatedException, ServerDisconnectedError, ClientConnectorError),
         max_tries=2,
         on_backoff=relogin,
     )
@@ -397,7 +429,7 @@ class OverkizClient:
 
     @backoff.on_exception(
         backoff.expo,
-        (NotAuthenticatedException, ServerDisconnectedError),
+        (NotAuthenticatedException, ServerDisconnectedError, ClientConnectorError),
         max_tries=2,
         on_backoff=relogin,
     )
@@ -417,7 +449,7 @@ class OverkizClient:
 
     @backoff.on_exception(
         backoff.expo,
-        (NotAuthenticatedException, ServerDisconnectedError),
+        (NotAuthenticatedException, ServerDisconnectedError, ClientConnectorError),
         max_tries=2,
         on_backoff=relogin,
     )
@@ -441,7 +473,7 @@ class OverkizClient:
 
     @backoff.on_exception(
         backoff.expo,
-        (NotAuthenticatedException, ServerDisconnectedError),
+        (NotAuthenticatedException, ServerDisconnectedError, ClientConnectorError),
         max_tries=2,
         on_backoff=relogin,
     )
@@ -465,7 +497,7 @@ class OverkizClient:
 
     @backoff.on_exception(
         backoff.expo,
-        (NotAuthenticatedException, ServerDisconnectedError),
+        (NotAuthenticatedException, ServerDisconnectedError, ClientConnectorError),
         max_tries=2,
         on_backoff=relogin,
     )
@@ -479,7 +511,10 @@ class OverkizClient:
         return execution_history
 
     @backoff.on_exception(
-        backoff.expo, NotAuthenticatedException, max_tries=2, on_backoff=relogin
+        backoff.expo,
+        (NotAuthenticatedException, ServerDisconnectedError, ClientConnectorError),
+        max_tries=2,
+        on_backoff=relogin,
     )
     async def get_device_definition(self, deviceurl: str) -> JSON | None:
         """
@@ -492,7 +527,10 @@ class OverkizClient:
         return response.get("definition")
 
     @backoff.on_exception(
-        backoff.expo, NotAuthenticatedException, max_tries=2, on_backoff=relogin
+        backoff.expo,
+        (NotAuthenticatedException, ServerDisconnectedError, ClientConnectorError),
+        max_tries=2,
+        on_backoff=relogin,
     )
     async def get_state(self, deviceurl: str) -> list[State]:
         """
@@ -506,7 +544,10 @@ class OverkizClient:
         return state
 
     @backoff.on_exception(
-        backoff.expo, NotAuthenticatedException, max_tries=2, on_backoff=relogin
+        backoff.expo,
+        (NotAuthenticatedException, ServerDisconnectedError, ClientConnectorError),
+        max_tries=2,
+        on_backoff=relogin,
     )
     async def refresh_states(self) -> None:
         """
@@ -514,6 +555,20 @@ class OverkizClient:
         """
         await self.__post("setup/devices/states/refresh")
 
+    @backoff.on_exception(
+        backoff.expo,
+        (NotAuthenticatedException, ServerDisconnectedError, ClientConnectorError),
+        max_tries=2,
+        on_backoff=relogin,
+    )
+    async def refresh_device_states(self, deviceurl: str) -> None:
+        """
+        Ask the box to refresh all states of the given device for protocols supporting that operation
+        """
+        await self.__post(
+            f"setup/devices/{urllib.parse.quote_plus(deviceurl)}/states/refresh"
+        )
+
     @backoff.on_exception(backoff.expo, TooManyConcurrentRequestsException, max_tries=5)
     async def register_event_listener(self) -> str:
         """
@@ -533,7 +588,10 @@ class OverkizClient:
 
     @backoff.on_exception(backoff.expo, TooManyConcurrentRequestsException, max_tries=5)
     @backoff.on_exception(
-        backoff.expo, NotAuthenticatedException, max_tries=2, on_backoff=relogin
+        backoff.expo,
+        (NotAuthenticatedException, ServerDisconnectedError, ClientConnectorError),
+        max_tries=2,
+        on_backoff=relogin,
     )
     @backoff.on_exception(
         backoff.expo,
@@ -564,7 +622,10 @@ class OverkizClient:
         self.event_listener_id = None
 
     @backoff.on_exception(
-        backoff.expo, NotAuthenticatedException, max_tries=2, on_backoff=relogin
+        backoff.expo,
+        (NotAuthenticatedException, ServerDisconnectedError, ClientConnectorError),
+        max_tries=2,
+        on_backoff=relogin,
     )
     async def get_current_execution(self, exec_id: str) -> Execution:
         """Get an action group execution currently running"""
@@ -574,7 +635,10 @@ class OverkizClient:
         return execution
 
     @backoff.on_exception(
-        backoff.expo, NotAuthenticatedException, max_tries=2, on_backoff=relogin
+        backoff.expo,
+        (NotAuthenticatedException, ServerDisconnectedError, ClientConnectorError),
+        max_tries=2,
+        on_backoff=relogin,
     )
     async def get_current_executions(self) -> list[Execution]:
         """Get all action groups executions currently running"""
@@ -584,7 +648,10 @@ class OverkizClient:
         return executions
 
     @backoff.on_exception(
-        backoff.expo, NotAuthenticatedException, max_tries=2, on_backoff=relogin
+        backoff.expo,
+        (NotAuthenticatedException, ServerDisconnectedError, ClientConnectorError),
+        max_tries=2,
+        on_backoff=relogin,
     )
     async def get_api_version(self) -> str:
         """Get the API version (local only)"""
@@ -594,7 +661,10 @@ class OverkizClient:
 
     @backoff.on_exception(backoff.expo, TooManyExecutionsException, max_tries=10)
     @backoff.on_exception(
-        backoff.expo, NotAuthenticatedException, max_tries=2, on_backoff=relogin
+        backoff.expo,
+        (NotAuthenticatedException, ServerDisconnectedError, ClientConnectorError),
+        max_tries=2,
+        on_backoff=relogin,
     )
     async def execute_command(
         self,
@@ -612,7 +682,7 @@ class OverkizClient:
 
     @backoff.on_exception(
         backoff.expo,
-        (NotAuthenticatedException, ServerDisconnectedError),
+        (NotAuthenticatedException, ServerDisconnectedError, ClientConnectorError),
         max_tries=2,
         on_backoff=relogin,
     )
@@ -621,7 +691,10 @@ class OverkizClient:
         await self.__delete(f"/exec/current/setup/{exec_id}")
 
     @backoff.on_exception(
-        backoff.expo, NotAuthenticatedException, max_tries=2, on_backoff=relogin
+        backoff.expo,
+        (NotAuthenticatedException, ServerDisconnectedError, ClientConnectorError),
+        max_tries=2,
+        on_backoff=relogin,
     )
     async def execute_commands(
         self,
@@ -639,7 +712,7 @@ class OverkizClient:
 
     @backoff.on_exception(
         backoff.expo,
-        (NotAuthenticatedException, ServerDisconnectedError),
+        (NotAuthenticatedException, ServerDisconnectedError, ClientConnectorError),
         max_tries=2,
         on_backoff=relogin,
     )
@@ -650,7 +723,7 @@ class OverkizClient:
 
     @backoff.on_exception(
         backoff.expo,
-        (NotAuthenticatedException, ServerDisconnectedError),
+        (NotAuthenticatedException, ServerDisconnectedError, ClientConnectorError),
         max_tries=2,
         on_backoff=relogin,
     )
@@ -662,7 +735,7 @@ class OverkizClient:
 
     @backoff.on_exception(
         backoff.expo,
-        (NotAuthenticatedException, ServerDisconnectedError),
+        (NotAuthenticatedException, ServerDisconnectedError, ClientConnectorError),
         max_tries=2,
         on_backoff=relogin,
     )
@@ -677,7 +750,7 @@ class OverkizClient:
 
     @backoff.on_exception(
         backoff.expo,
-        (NotAuthenticatedException, ServerDisconnectedError),
+        (NotAuthenticatedException, ServerDisconnectedError, ClientConnectorError),
         max_tries=2,
         on_backoff=relogin,
     )
@@ -697,7 +770,7 @@ class OverkizClient:
 
     @backoff.on_exception(
         backoff.expo,
-        (NotAuthenticatedException, ServerDisconnectedError),
+        (NotAuthenticatedException, ServerDisconnectedError, ClientConnectorError),
         max_tries=2,
         on_backoff=relogin,
     )
@@ -715,7 +788,7 @@ class OverkizClient:
 
     @backoff.on_exception(
         backoff.expo,
-        (NotAuthenticatedException, ServerDisconnectedError),
+        (NotAuthenticatedException, ServerDisconnectedError, ClientConnectorError),
         max_tries=2,
         on_backoff=relogin,
     )
@@ -729,7 +802,10 @@ class OverkizClient:
         return True
 
     @backoff.on_exception(
-        backoff.expo, NotAuthenticatedException, max_tries=2, on_backoff=relogin
+        backoff.expo,
+        (NotAuthenticatedException, ServerDisconnectedError, ClientConnectorError),
+        max_tries=2,
+        on_backoff=relogin,
     )
     async def execute_scenario(self, oid: str) -> str:
         """Execute a scenario"""
@@ -738,7 +814,7 @@ class OverkizClient:
 
     @backoff.on_exception(
         backoff.expo,
-        (NotAuthenticatedException, ServerDisconnectedError),
+        (NotAuthenticatedException, ServerDisconnectedError, ClientConnectorError),
         max_tries=2,
         on_backoff=relogin,
     )
@@ -747,6 +823,64 @@ class OverkizClient:
         response = await self.__post(f"exec/schedule/{oid}/{timestamp}")
         return cast(str, response["triggerId"])
 
+    @backoff.on_exception(
+        backoff.expo,
+        (NotAuthenticatedException, ServerDisconnectedError, ClientConnectorError),
+        max_tries=2,
+        on_backoff=relogin,
+    )
+    async def get_setup_options(self) -> list[Option]:
+        """
+        This operation returns all subscribed options of a given setup.
+        Per-session rate-limit : 1 calls per 1d period for this particular operation (bulk-load)
+        Access scope : Full enduser API access (enduser/*)
+        """
+        response = await self.__get("setup/options")
+        options = [Option(**o) for o in humps.decamelize(response)]
+
+        return options
+
+    @backoff.on_exception(
+        backoff.expo,
+        (NotAuthenticatedException, ServerDisconnectedError, ClientConnectorError),
+        max_tries=2,
+        on_backoff=relogin,
+    )
+    async def get_setup_option(self, option: str) -> Option | None:
+        """
+        This operation returns the selected subscribed option of a given setup.
+        For example `developerMode-{gateway_id}` to understand if developer mode is on.
+        """
+        response = await self.__get(f"setup/options/{option}")
+
+        if response:
+            return Option(**humps.decamelize(response))
+
+        return None
+
+    @backoff.on_exception(
+        backoff.expo,
+        (NotAuthenticatedException, ServerDisconnectedError, ClientConnectorError),
+        max_tries=2,
+        on_backoff=relogin,
+    )
+    async def get_setup_option_parameter(
+        self, option: str, parameter: str
+    ) -> OptionParameter | None:
+        """
+        This operation returns the selected parameters of a given setup and option.
+        For example `developerMode-{gateway_id}` and `gatewayId` to understand if developer mode is on.
+
+        If the option is not available, an OverkizException will be thrown.
+        If the parameter is not available you will receive None.
+        """
+        response = await self.__get(f"setup/options/{option}/{parameter}")
+
+        if response:
+            return OptionParameter(**humps.decamelize(response))
+
+        return None
+
     async def __get(self, path: str) -> Any:
         """Make a GET request to the OverKiz API"""
         headers = {}
@@ -758,6 +892,7 @@ class OverkizClient:
         async with self.session.get(
             f"{self.server.endpoint}{path}",
             headers=headers,
+            ssl_context=self._ssl_context,
         ) as response:
             await self.check_response(response)
             return await response.json()
@@ -773,7 +908,11 @@ class OverkizClient:
             headers["Authorization"] = f"Bearer {self._access_token}"
 
         async with self.session.post(
-            f"{self.server.endpoint}{path}", data=data, json=payload, headers=headers
+            f"{self.server.endpoint}{path}",
+            data=data,
+            json=payload,
+            headers=headers,
+            ssl_context=self._ssl_context,
         ) as response:
             await self.check_response(response)
             return await response.json()
@@ -788,7 +927,9 @@ class OverkizClient:
             headers["Authorization"] = f"Bearer {self._access_token}"
 
         async with self.session.delete(
-            f"{self.server.endpoint}{path}", headers=headers
+            f"{self.server.endpoint}{path}",
+            headers=headers,
+            ssl_context=self._ssl_context,
         ) as response:
             await self.check_response(response)
 
@@ -802,14 +943,28 @@ class OverkizClient:
             result = await response.json(content_type=None)
         except JSONDecodeError as error:
             result = await response.text()
-            if "Server is down for maintenance" in result:
+
+            if "is down for maintenance" in result:
                 raise MaintenanceException("Server is down for maintenance") from error
-            raise Exception(
+
+            if response.status == 503:
+                raise ServiceUnavailableException(result) from error
+
+            raise OverkizException(
                 f"Unknown error while requesting {response.url}. {response.status} - {result}"
             ) from error
 
         if result.get("errorCode"):
-            message = result.get("error")
+            # Error messages between cloud and local Overkiz servers can be slightly different
+            # To make it easier to have a strict match for these errors, we remove the double quotes and the period at the end.
+
+            if message := result.get("error"):
+                message = message.strip(
+                    '".'
+                )  # Change to .removesuffix when Python 3.8 support is dropped
+            else:
+                # An error message can have an empty (None) message
+                message = ""
 
             # {"errorCode": "AUTHENTICATION_ERROR",
             # "error": "Too many requests, try again later : login with xxx@xxx.tld"}
@@ -825,7 +980,7 @@ class OverkizClient:
                 raise NotAuthenticatedException(message)
 
             # {"error":"Missing authorization token.","errorCode":"RESOURCE_ACCESS_DENIED"}
-            if message == "Missing authorization token.":
+            if message == "Missing authorization token":
                 raise MissingAuthorizationTokenException(message)
 
             # {"error": "Server busy, please try again later. (Too many executions)"}
@@ -848,13 +1003,14 @@ class OverkizClient:
             if message == "too many concurrent requests":
                 raise TooManyConcurrentRequestsException(message)
 
+            # {'errorCode': 'EXEC_QUEUE_FULL', 'error': 'Execution queue is full on gateway: #xxx-yyyy-zzzz (soft limit: 10)'}
+            if "Execution queue is full on gateway" in message:
+                raise ExecutionQueueFullException(message)
+
             if message == "Cannot use JSESSIONID and bearer token in same request":
                 raise SessionAndBearerInSameRequestException(message)
 
-            if (
-                message
-                == "Too many attempts with an invalid token, temporarily banned."
-            ):
+            if message == "Too many attempts with an invalid token, temporarily banned":
                 raise TooManyAttemptsBannedException(message)
 
             if "Invalid token : " in message:
@@ -867,14 +1023,15 @@ class OverkizClient:
                 raise UnknownUserException(message)
 
             # {"error":"Unknown object.","errorCode":"UNSPECIFIED_ERROR"}
-            if message == "Unknown object.":
+            if message == "Unknown object":
                 raise UnknownObjectException(message)
 
             # {'errorCode': 'RESOURCE_ACCESS_DENIED', 'error': 'Access denied to gateway #1234-5678-1234 for action ADD_TOKEN'}
             if "Access denied to gateway" in message:
                 raise AccessDeniedToGatewayException(message)
 
-        raise Exception(message if message else result)
+        # General Overkiz exception
+        raise OverkizException(result)
 
     async def _refresh_token_if_expired(self) -> None:
         """Check if token is expired and request a new one."""
