@@ -1,8 +1,16 @@
 import logging
-from enum import Enum, IntEnum, unique
+import sys
+from enum import IntEnum, unique
 
 _LOGGER = logging.getLogger(__name__)
 
+# Since we support Python versions lower than 3.11, we use
+# a backport for StrEnum when needed.
+if sys.version_info >= (3, 11):
+    from enum import StrEnum
+else:
+    from backports.strenum import StrEnum
+
 
 @unique
 class ProductType(IntEnum):
@@ -126,6 +134,7 @@ class FailureType(IntEnum):
     PRIORITY_LOCK__NO_SECURITY_DEVICE = 192
     PRIORITY_LOCK__DEAD_SENSOR = 193
     PRIORITY_LOCK__UNKNOWN_ERROR = 194
+    CMDDEPRECATED = 197
     DBUS_ERROR = 200
     DBUS_NO_MEMORY = 201
     DBUS_SERVICE_UNKNOWN = 202
@@ -231,7 +240,10 @@ class FailureType(IntEnum):
     PEER_DOWN = 10005
     GATEWAY_BUFFER_OVERFLOW = 10006
     UNKNOWN_DETAILED_ERROR = 10007
+    TIME_OUT_ON_TRANSMIT = 20001
+    TIME_OUT_ON_TRANSMITTED_COMMAND = 20002
     TIME_OUT_ON_COMMAND_PROGRESS = 20003
+    DEVICE_NO_ANSWER = 60004
 
     @classmethod
     def _missing_(cls, value):  # type: ignore
@@ -240,7 +252,7 @@ class FailureType(IntEnum):
 
 
 @unique
-class EventName(str, Enum):
+class EventName(StrEnum):
     UNKNOWN = "Unknown"
     ACTION_GROUP_CREATED = "ActionGroupCreatedEvent"
     ACTION_GROUP_DELETED = "ActionGroupDeletedEvent"
@@ -249,6 +261,7 @@ class EventName(str, Enum):
     CALENDAR_DAY_UPDATED = "CalendarDayUpdatedEvent"
     CALENDAR_RULE_CREATED = "CalendarRuleCreatedEvent"
     CALENDAR_RULE_DELETED = "CalendarRuleDeletedEvent"
+    CALENDAR_RULE_UPDATED = "CalendarRuleUpdatedEvent"
     CAMERA_DISCOVERED = "CameraDiscoveredEvent"
     CAMERA_DISCOVER_FAILED = "CameraDiscoverFailedEvent"
     CAMERA_UPLOAD_PHOTO = "CameraUploadPhotoEvent"
@@ -333,6 +346,7 @@ class EventName(str, Enum):
     MAIN_ACCOUNT_UPDATED = "MainAccountUpdatedEvent"
     NEW_DEVICE_FOUND = "NewDeviceFoundEvent"
     NOTIFICATION_EVENT_LOG = "NotificationEventLogEvent"
+    OGP = "OGPEvent"
     OPENDOORS_DISCOVER_COMPLETED = "OpenDoorsDiscoverCompletedEvent"
     OPENDOORS_DISCOVER_FAILED = "OpenDoorsDiscoverFailedEvent"
     OPENDOORS_GENERATE_OAUTH_TOKENS_COMPLETED = (
