@@ -1,27 +1,42 @@
-from enum import Enum, unique
+import sys
+from enum import unique
+
+# Since we support Python versions lower than 3.11, we use
+# a backport for StrEnum when needed.
+if sys.version_info >= (3, 11):
+    from enum import StrEnum
+else:
+    from backports.strenum import StrEnum
 
 
 @unique
-class OverkizAttribute(str, Enum):
+class OverkizAttribute(StrEnum):
     """Device attributes used by Overkiz."""
 
     CORE_FIRMWARE_REVISION = "core:FirmwareRevision"
     CORE_MANUFACTURER = "core:Manufacturer"
+    CORE_MANUFACTURER_REFERENCE = "core:ManufacturerReference"
     CORE_MAX_SETTABLE_VALUE = "core:MaxSettableValue"
     CORE_MEASURED_VALUE_TYPE = "core:MeasuredValueType"
     CORE_MIN_SETTABLE_VALUE = "core:MinSettableValue"
     CORE_ELECTRIC_POWER_CONSUMPTION_STATE_MEASURED_VALUE_TYPE = (
         "core:ElectricPowerConsumptionStateMeasuredValueType"
     )
+    CORE_SUPPORTED_ALIASES = "core:SupportedAliases"
+    CORE_SUPPORTED_OPTIONS = "core:SupportedOptions"
+    CORE_TECHNOLOGY = "core:Technology"
     HOMEKIT_SETUP_CODE = "homekit:SetupCode"
+    OGP_FEATURES = "ogp:Features"
 
 
 @unique
-class OverkizState(str, Enum):
+class OverkizState(StrEnum):
     """Device states used by Overkiz."""
 
     CORE_ACTIVE_ZONES = "core:ActiveZonesState"
+    CORE_ACTIVATED_OPTIONS = "core:ActivatedOptionsState"
     CORE_ASSEMBLY = "core:AssemblyState"
+    CORE_AUTO_MANU_MODE = "core:AutoManuModeState"
     CORE_AVAILABILITY = "core:AvailabilityState"
     CORE_BATTERY = "core:BatteryState"
     CORE_BATTERY_LEVEL = "core:BatteryLevelState"
@@ -72,6 +87,7 @@ class OverkizState(str, Enum):
     CORE_GAS_CONSUMPTION = "core:GasConsumptionState"
     CORE_GAS_DETECTION = "core:GasDetectionState"
     CORE_GREEN_COLOR_INTENSITY = "core:GreenColorIntensityState"
+    CORE_HOLIDAYS_MODE = "core:HolidaysModeState"
     CORE_HEATING_ON_OFF = "core:HeatingOnOffState"
     CORE_HEATING_STATUS = "core:HeatingStatusState"
     CORE_HEATING_TARGET_TEMPERATURE = "core:HeatingTargetTemperatureState"
@@ -109,13 +125,14 @@ class OverkizState(str, Enum):
     CORE_OPEN_CLOSED_PEDESTRIAN = "core:OpenClosedPedestrianState"
     CORE_OPEN_CLOSED_UNKNOWN = "core:OpenClosedUnknownState"
     CORE_OPEN_CLOSED_VALVE = "core:OpenClosedValveState"
+    CORE_OPEN_WINDOW_DETECTION_ACTIVATION = "core:OpenWindowDetectionActivationState"
     CORE_OPERATING_MODE = "core:OperatingModeState"
     CORE_PEDESTRIAN_POSITION = "core:PedestrianPositionState"
     CORE_POWER_HEAT_ELECTRICAL_IN = "core:PowerHeatElectricalInState"
     CORE_POWER_SUPPLY_UP_DOWN = "core:PowerSupplyUpDownState"
     CORE_PRIORITY_LOCK_TIMER = "core:PriorityLockTimerState"
     CORE_PRODUCT_MODEL_NAME = "core:ProductModelNameState"
-    CORE_PROGRAMING_AVAILABLE_STATE = "core:ProgrammingAvailableState"
+    CORE_PROGRAMMING_AVAILABLE = "core:ProgrammingAvailableState"
     CORE_RAIN = "core:RainState"
     CORE_RED_COLOR_INTENSITY = "core:RedColorIntensityState"
     CORE_RELATIVE_HUMIDITY = "core:RelativeHumidityState"
@@ -148,6 +165,7 @@ class OverkizState(str, Enum):
     HLRRWIFI_LEAVE_HOME = "hlrrwifi:LeaveHomeState"
     HLRRWIFI_MAIN_OPERATION = "hlrrwifi:MainOperationState"
     HLRRWIFI_MODE_CHANGE = "hlrrwifi:ModeChangeState"
+    HLRRWIFI_OUTDOOR_TEMPERATURE = "hlrrwifi:OutdoorTemperatureState"
     HLRRWIFI_ROOM_TEMPERATURE = "hlrrwifi:RoomTemperatureState"
     HLRRWIFI_SWING = "hlrrwifi:SwingState"
     INTERNAL_CURRENT_ALARM_MODE = "internal:CurrentAlarmModeState"
@@ -155,6 +173,7 @@ class OverkizState(str, Enum):
     INTERNAL_TARGET_ALARM_MODE = "internal:TargetAlarmModeState"
     IO_AIR_DEMAND_MODE = "io:AirDemandModeState"
     IO_AWAY_MODE_DURATION = "io:AwayModeDurationState"
+    IO_BYPASS_ACTIVATION = "io:ByPassActivationState"
     IO_DEROGATION_HEATING_MODE = "io:DerogationHeatingModeState"
     IO_DEROGATION_REMAINING_TIME = "io:DerogationRemainingTimeState"
     IO_DHW_ABSENCE_MODE = "io:DHWAbsenceModeState"
@@ -165,6 +184,7 @@ class OverkizState(str, Enum):
     IO_FORCE_HEATING = "io:ForceHeatingState"
     IO_HEAT_PUMP_OPERATING_TIME = "io:HeatPumpOperatingTimeState"
     IO_INLET_ENGINE = "io:InletEngineState"
+    IO_LOCK_KEY_ACTIVATION = "io:LockKeyActivationState"
     IO_MEMORIZED_SIMPLE_VOLUME = "io:MemorizedSimpleVolumeState"
     IO_MIDDLE_WATER_TEMPERATURE = "io:MiddleWaterTemperatureState"
     IO_MODEL = "io:ModelState"
@@ -181,6 +201,7 @@ class OverkizState(str, Enum):
     IO_SENSOR_ROOM = "io:SensorRoomState"
     IO_TARGET_HEATING_LEVEL = "io:TargetHeatingLevelState"
     IO_TOWEL_DRYER_TEMPORARY_STATE = "io:TowelDryerTemporaryStateState"
+    IO_VALVE_INSTALLATION_MODE = "io:ValveInstallationModeState"
     IO_VENTILATION_CONFIGURATION_MODE = "io:VentilationConfigurationModeState"
     IO_VENTILATION_MODE = "io:VentilationModeState"
     IO_VIBRATION_DETECTED = "io:VibrationDetectedState"
@@ -224,8 +245,10 @@ class OverkizState(str, Enum):
     OVP_LEAVE_HOME = "ovp:LeaveHomeState"
     OVP_MAIN_OPERATION = "ovp:MainOperationState"
     OVP_MODE_CHANGE = "ovp:ModeChangeState"
+    OVP_OUTDOOR_TEMPERATURE = "ovp:OutdoorTemperatureState"
     OVP_ROOM_TEMPERATURE = "ovp:RoomTemperatureState"
     OVP_SWING = "ovp:SwingState"
+    OVP_TEMPERATURE_CHANGE = "ovp:TemperatureChangeState"
     RAMSES_RAMSES_OPERATING_MODE = "ramses:RAMSESOperatingModeState"
     RTDS_CONTROLLER_BATTERY = "rtds:ControllerBatteryState"
     RTDS_CONTROLLER_BIP = "rtds:ControllerBipState"
