@@ -1,11 +1,19 @@
 import logging
-from enum import Enum, unique
+import sys
+from enum import unique
 
 _LOGGER = logging.getLogger(__name__)
 
+# Since we support Python versions lower than 3.11, we use
+# a backport for StrEnum when needed.
+if sys.version_info >= (3, 11):
+    from enum import StrEnum
+else:
+    from backports.strenum import StrEnum
+
 
 @unique
-class UIClass(str, Enum):
+class UIClass(StrEnum):
     # A list of all defined UI classes from /reference/ui/classes
     ADJUSTABLE_SLATS_ROLLER_SHUTTER = "AdjustableSlatsRollerShutter"
     AIR_SENSOR = "AirSensor"
@@ -82,7 +90,7 @@ class UIClass(str, Enum):
 
 
 @unique
-class UIWidget(str, Enum):
+class UIWidget(StrEnum):
     # A list of all defined UI widgets from /reference/ui/widgets
     AIR_FLOW_SENSOR = "AirFlowSensor"
     AIR_QUALITY_SENSOR = "AirQualitySensor"
@@ -132,6 +140,7 @@ class UIWidget(str, Enum):
     )
     CUMULATIVE_WATER_CONSUMPTION_SENSOR = "CumulativeWaterConsumptionSensor"
     CURTAIN_TRACK_UNO = "CurtainTrackUno"
+    CYCLIC_GARAGE_DOOR = "CyclicGarageDoor"
     CYCLIC_GENERIC = "CyclicGeneric"
     DHW_SET_POINT = "DHWSetPoint"
     DE_DIETRICH_BOILER = "DeDietrichBoiler"
@@ -150,6 +159,7 @@ class UIWidget(str, Enum):
     DIMMER_RGBCOLOURED_LIGHT = "DimmerRGBColouredLight"
     DIMPLEX_VENTILATION_INLET_OUTLET = "DimplexVentilationInletOutlet"
     DISCRETE_EXTERIOR_HEATING = "DiscreteExteriorHeating"
+    DISCRETE_GATE_WITH_PEDESTRIAN_POSITION = "DiscreteGateWithPedestrianPosition "
     DISCRETE_POSITIONABLE_GARAGE_DOOR = "DiscretePositionableGarageDoor"
     DISCRETE_POSITIONABLE_GATE = "DiscretePositionableGate"
     DOCK = "Dock"
@@ -159,6 +169,7 @@ class UIWidget(str, Enum):
     DROP_ARM_AWNING = "DropArmAwning"
     DYNAMIC_AIR_VENT = "DynamicAirVent"
     DYNAMIC_ALARM = "DynamicAlarm"
+    DYNAMIC_AUDIO_PLAYER = "DynamicAudioPlayer"
     DYNAMIC_AWNING = "DynamicAwning"
     DYNAMIC_BRIDGE = "DynamicBridge"
     DYNAMIC_CIRCUIT_BREAKER = "DynamicCircuitBreaker"
@@ -178,6 +189,7 @@ class UIWidget(str, Enum):
     DYNAMIC_OVEN = "DynamicOven"
     DYNAMIC_PERGOLA = "DynamicPergola"
     DYNAMIC_RAIN_SENSOR = "DynamicRainSensor"
+    DYNAMIC_ROLLER_SHUTTER = "DynamicRollerShutter"
     DYNAMIC_SCREEN = "DynamicScreen"
     DYNAMIC_SHUTTER = "DynamicShutter"
     DYNAMIC_TEMPERATURE_SENSOR = "DynamicTemperatureSensor"
@@ -217,6 +229,7 @@ class UIWidget(str, Enum):
     HITACHI_DHW = "HitachiDHW"
     HITACHI_SWIMMING_POOL = "HitachiSwimmingPool"
     HITACHI_THERMOSTAT = "HitachiThermostat"
+    HLRR_WIFI_BRIDGE = "HLRRWifiBridge"
     HOMEKIT_STACK = "HomekitStack"
     HUE_BRIDGE = "HueBridge"
     IOGENERIC = "IOGeneric"
