@@ -221,7 +221,6 @@ class Device:
 
 @define(init=False, kw_only=True)
 class StateDefinition:
-
     qualified_name: str
     type: str | None = None
     values: list[str] | None = None
@@ -658,7 +657,7 @@ class HistoryExecution:
     event_time: int
     owner: str = field(repr=obfuscate_email)
     source: str
-    end_time: int
+    end_time: int | None = None
     effective_start_time: int | None = None
     duration: int
     label: str | None = None
@@ -676,7 +675,7 @@ class HistoryExecution:
         event_time: int,
         owner: str,
         source: str,
-        end_time: int,
+        end_time: int | None = None,
         effective_start_time: int | None = None,
         duration: int,
         label: str | None = None,
@@ -797,3 +796,36 @@ class LocalToken:
     gateway_creation_time: int
     uuid: str
     scope: str
+
+
+@define(kw_only=True)
+class OptionParameter:
+    name: str
+    value: str
+
+
+@define(init=False, kw_only=True)
+class Option:
+    creation_time: int
+    last_update_time: int
+    option_id: str
+    start_date: int
+    parameters: list[OptionParameter] | None
+
+    def __init__(
+        self,
+        *,
+        creation_time: int,
+        last_update_time: int,
+        option_id: str,
+        start_date: int,
+        parameters: list[dict[str, Any]] | None,
+        **_: Any,
+    ) -> None:
+        self.creation_time = creation_time
+        self.last_update_time = last_update_time
+        self.option_id = option_id
+        self.start_date = start_date
+        self.parameters = (
+            [OptionParameter(**p) for p in parameters] if parameters else []
+        )
