@@ -17,4 +17,4 @@ DATA_TYPE_TO_PYTHON: dict[DataType, Callable[[Any], StateType]] = {
     DataType.JSON_OBJECT: json.loads,
 }
 
-JSON = Union[Dict[str, Any], List[Dict[str, Any]]]
+JSON = Union[Dict[str, Any], List[Dict[str, Any]]]  # pylint: disable=invalid-name
